""" goulash.decorators
"""
import inspect

class arg_types(object):
    """ A decorator which enforces the rule that all arguments must be
        of type .  All keyword arguments are ignored. Throws ArgTypeError
        when expectations are violated.

        Example usage follows:

          @arg_types(int, float)
          def sum(*args): pass
    """

    class ArgTypeError(TypeError): pass

    def __init__(self, *args):
        err = 'all arguments to arg_types() should be types, got {0}'
        assert all([inspect.isclass(a) for a in args]), err.format(args)
        self.types = args

    def __call__(self, fxn):
        self.fxn = fxn
        def wrapped(*args, **kargs):
            for a in args:
                if not isinstance(a, self.types):
                    raise self.ArgTypeError("{0} (type={1}) is not in {2}".format(
                        a, type(a), self.types))
            return self.fxn(*args, **kargs)
        return wrapped

class memoized_property(object):
    """
    A read-only @property that is only evaluated once.

    From: http://www.reddit.com/r/Python/comments/ejp25/cached_property_decorator_that_is_memory_friendly/
    """
    def __init__(self, fget, doc=None):
        self.fget = fget
        self.__doc__ = doc or fget.__doc__
        self.__name__ = fget.__name__

    def __get__(self, obj, cls):
        if obj is None:
            return self
        obj.__dict__[self.__name__] = result = self.fget(obj)
        return result



""" goulash.tests.test_allstatic
"""

import unittest

from goulash.allstatic import AllStaticMethods


class TestAllStatic(unittest.TestCase):

    def test_basic(self):
        class Testing(object):
            __metaclass__ = AllStaticMethods
            def foo(): return 'foo'
        self.assertEqual(Testing.foo(),'foo')



""" goulash.namespaces
"""
import types
from copy import copy

from inspect import isfunction, isclass, ismethod

class ValueNotFound(Exception): pass

def classname(thing):
    """ Return the fully-qualified class
        name of the specified class or object
    """
    try:
        return thing.__module__ + '.' + thing.__name__
    except:
        pass
    try:
        return thing.__module__ + '.' + thing.__class__.__name__
    except:
        _ty = str(type(thing))
        raise AssertionError('Must pass class or instance object, got'+_ty)

class NamespaceTests:
    """ Various boolean tests over objects, packaged thus to be resuable
        TODO: NamespaceTests(StaticMethodsOnly)
    """

    @staticmethod
    def is_property(obj):
        return isinstance(obj, property)

    @staticmethod
    def name_startswith(obj, pattern):
        """ """
        return hasattr(obj, '__name__') and obj.__name__.startswith(pattern)

    @staticmethod
    def dictionaryish(obj):
        """ """
        return isinstance(obj, dict) and \
               all([isinstance(k, basestring) for k in obj.keys()])
#type#hasattr(obj, 'keys') and hasattr(obj.keys,'__call__')

    @staticmethod
    def is_unittest_testcase_class(obj):
        """ """
        import unittest
        return isclass(obj) and issubclass(obj, unittest.TestCase)

def grab(obj, k):
    return getattr(obj, k)

class Namespace(object):
    """ NamespacePartion: introspective operations over dictionary-like objects

        NOTE: By default, all operations return dictionaries. Set
            dictionaries to False and you can get back another
            partion object.

        NOTE: This does not work in-place. (see the copy import up there?)
    """


    def __repr__(self):
        return "Namespace({0})".format(str(self.obj))

    ## dictionary compatability
    ############################################################################
    def items(self): return self.namespace.items()
    def values(self): return self.namespace.values()
    def keys(self): return reversed(sorted(self.namespace.keys()))
    def __iter__(self): return iter(self.namespace)
    def __getitem__(self,name): return self.namespace[name]
    def copy(self):
        """ This can fail for a variety of reasons involving
            thread safety, etc.. hopefully this approach is
            somewhat reasonable for the average case though
        """
        try:
            return copy(self.namespace)
        except TypeError:
            return dict([[name, self.namespace[name]] for name in self.namespace])

    ## pseudo-dictionary and/or set() compatability
    ############################################################################
    def intersection(self, other):
        if isinstance(other, (dict, NSPart)):
            other = getattr(other, 'namespace', other)
            result = [ [k,self[k]] for k in self.namespace if k in other]
        else:
            raise RuntimeError,'niy'
        result = dict(result)
        return result if self.dictionaries else self.__class__(result, original=self)
    def __add__(self, other):
        """ Update this namespace with another.

            works for all combinations of dict+namespace,
            namespace+dict, namespace+namespace.  if any of the
            constituents are or want to return dictionaries,
            return type will be dictionaries.
        """
        out = copy(self.namespace)
        if isinstance(other, Namespace):
            out.update(other.namespace)
        elif isinstance(other, dict):
            out.update(other)
            return out
        if self.dictionaries or other.dictionaries:
            return out
        else:
            return Namespace(out, dictionaries=False)

    def __init__(self, obj, dictionaries=True, original=None):
        """ """
        self.dictionaries = dictionaries

        if original is not None and isinstance(obj, dict):
            assert isinstance(original, Namespace)
            self.namespace = obj
            self.obj = original.obj
            self.dictionaries = original.dictionaries

        elif not NamespaceTests.dictionaryish(obj):
            if isinstance(obj,dict):
                err = ("You gave a dictionary, but maybe the keys aren't strings?")
                #warning.warn(err)
                self.obj=obj
                self.namespace={}
                return
            if not hasattr(obj, '__dict__'):
                err = ("Namespace Partitioner really expects something "
                       "like a dictionary, got {0}".format(type(obj).__name__))
                raise TypeError, err
            namespace = {}
            if not isinstance(obj, dict):
                namespace.update(**dict([[k, grab(obj,k)] for k in dir(obj)]))
            else:
                namespace = obj
            self.namespace=namespace
            self.obj=obj
        else:
            self.namespace = obj
            self.obj = obj

    ## core introspection stuff
    ############################################################################
    @property
    def nonprivate(self):
        return self._clean()

    @property
    def properties(self):
        this = self if isclass(self.obj) else \
               self.__class__(self.obj.__class__)
        return this.generic(NamespaceTests.is_property)

    @property
    def unittest_testcases(self):
        """ Filter unittest test cases """
        return self.generic(NamespaceTests.is_unittest_testcase_class)

    @property
    def django_testcases(self):
        """ Filter django test cases """
        return self.generic(NamespaceTests.is_django_testcase_class)

    @property
    def functions(self):
        """ Filter functions """
        return self.generic(isfunction)

    @property
    def methods(self):
        """ Filter methods """
        return self.generic(ismethod)

    @property
    def private(self):
        return self.startswith('_')

    @property
    def data(self):
        """ no methods, no private stuff
            TODO: no "complex" stuff e.g. classobj
        """
        tmp = NSPart(self.copy())
        tmp.dictionaries = False
        tmp = tmp.nonprivate
        result = [ [x, self.namespace[x]] for x in tmp if x not in tmp.methods ]
        result = [ [x[0],x[1]] for x in result if not isclass(x[1]) ]
        result = dict(result)
        return result if self.dictionaries else NSPart(result)

    ## generic methods
    ############################################################################
    @property
    def subobjects(self):
        """ just sugar. """
        return self

    def with_attr(self,name):
        result=[]
        for k, v in self.items():
            if hasattr(v,name):
                result.append([k,v])
        result = dict(result)
        return result if self.dictionaries else self.__class__(result, original=self)

    def _clean(self, pattern='_'):
        """ For dictionary-like objects we'll clean out names that start with
            pattern.. for generic objects, we'll turn them into namespace
            dictionaries and proceed.
        """
        namespace = copy(self.namespace)
        bad_names = [x for x in namespace.keys() if x.startswith(pattern)]
        [ namespace.pop(name) for name in bad_names ]
        return namespace if self.dictionaries else self.__class__(namespace, original=self)

    def startswith(self, string):
        return self.generic_key(lambda k: k.startswith(string))


    def generic(self, test, value_test=True):
        """ Partitions the namespace with respect to a test function.

                TODO: refactor this around inspect.getmemebers()
        """
        namespace = self.copy()
        for key, val in namespace.items():
            if value_test:
                if not test(val):
                    namespace.pop(key)
            else: #keytest
                if not test(key):
                    namespace.pop(key)
        return namespace if self.dictionaries else \
               self.__class__(namespace, original=self)

    def generic_key(self, test):
        return self.generic(test, value_test=False)

    def type_equal(self, thing):
        """ filter by type """
        _ty = ( type(thing).__name__=='type' and thing) or type(thing)
        return self.generic(lambda obj: type(obj)==_ty)

    def subclasses_of(self, thing, strict=True):
        """ filter by subclass """
        kls = thing
        if not isclass(thing):
            kls = thing.__class__
        test = lambda obj: issubclass(obj,kls)
        return self.generic(test)

    @property
    def class_variables(self):
        if not isclass(self.obj):
            return Namespace(self.obj.__class__).class_variables
        keys = set(self.namespace.keys())
        keys -= set(self.methods.keys())
        keys -= set(self.functions.keys())
        result = dict([[key,self.namespace[key]] for key in keys])
        return result if self.dictionaries else self.__class__(result, original=self)

    @property
    def locals(self):
        """ only things that are defined by this object or this object's
            class.. i.e. nothing from superclasses
        """
        keys = set(self.keys())
        kls = self.obj if isclass(self.obj) else self.obj.__class__
        bases = kls.__bases__
        # because some (misbehaving) __gettr__ hacks
        # could possibly return noniterable __bases__
        #try:
        base_ns = [dir(b) for b in bases]
        #except TypeError: result = {}
        #else:
        base_ns = set(reduce(lambda x,y: x+y, base_ns))
        keys = keys - base_ns
        result = dict([[k, self.namespace[k]] for k in keys])
        [result.pop(x,None) for x in ('__dict__','__module__','__weakref__')]
        return result if self.dictionaries else self.__class__(result, original=self)



# Begin aliases, shortcuts
################################################################################
NSPart = Namespace
clean_namespace = lambda namespace: Namespace(namespace).cleaned
Tests = NamespaceTests



""" test_os """

import unittest

from goulash import _os

class TestOS(unittest.TestCase):

    def setUp(self):
        pass

    def test_get_mounts(self):
        tmp = _os.get_mounts_by_type('proc')
        for mp in tmp:
            self.assertTrue('name' in mp)
            self.assertTrue('mount_point' in mp)



""" goulash.tests.test_heuristics
"""
import unittest
from goulash import heuristics as hx


class TestHeuristics(unittest.TestCase):

    def test_answer(self):
        self.assertTrue(hx.Answer(42))
        self.assertTrue(hx.Answer(True))
        self.assertFalse(hx.Answer(False))
        self.assertFalse(hx.Answer(0))

    def test_explained_answer(self):
        explanation = 'argument from authority'
        tmp = hx.ExplainedAnswer(True, explanation)
        self.assertTrue(tmp)
        self.assertEqual(tmp.explanation, explanation)
        actual = str(tmp)
        expected = '(ExplainedAnswer: argument from authority)'
        self.assertEqual(actual, expected)

    def test_not_applicable(self):
        tmp = hx.NotApplicable("heuristic does not apply!")
        self.assertFalse(tmp)
        tmp = hx.NotApplicable()
        self.assertFalse(tmp)

    def test_affirmative(self):
        tmp = hx.Affirmative("yup")
        self.assertTrue(tmp)

    def test_negative(self):
        tmp = hx.Negative("nope")
        self.assertFalse(tmp)



""" test_wrappers:

      tests for goulash.wrappers
"""

import unittest

from goulash.wrappers import JSONWrapper, DumbWrapper

class TestWrappers(unittest.TestCase):
    def setUp(self):
        self.data = dict(foo='bar')
        self.jwrapper = JSONWrapper(self.data)
        self.dwrapper = DumbWrapper(self.data)

    def test_json_wrapper_data(self):
        self.assertEqual(self.jwrapper.foo, 'bar')

    def test_json_wrapper_default(self):
        self.jwrapper.test=1
        self.assertEqual(self.jwrapper.test, 1)



""" goulash.tests.test_namespaces
"""

from unittest import TestCase, main

from goulash import Namespace
from goulash.tests.data import TestObject, ComplexTestObject

class TestRecursive(TestCase):
    def setUp(self):
        self.obj = TestObject()
        self.ns = Namespace(self. obj, dictionaries=False)

    def test_original(self):
        self.assertEqual(self.obj, self.ns.obj)
        self.assertEqual(self.obj, self.ns.private.obj)
        self.assertEqual(self.obj, self.ns.nonprivate.obj)

class TestBasic(TestCase):
    def setUp(self):
        self.test_obj = TestObject()
        self.test_obj2 = ComplexTestObject()
        self.test_ns = Namespace(self.test_obj)

    def test_subobject_with_attr(self):
        self.assertEqual(
            ['_private_method'],
            self.test_ns.subobjects.with_attr('test_attr').keys())

    def test_properties(self):
        self.assertEqual(self.test_ns.properties.values(),
                         [TestObject._some_private_property])

    def test_locals(self):
        test_ns2 = Namespace(self.test_obj2)
        tmp = test_ns2.locals
        self.assertEqual(tmp.keys(), ['my_class_variable'])

    def test_keys(self):
        diff = list(set(self.test_ns.keys()) - set(dir(self.test_obj)))
        self.assertFalse(diff)

    def test_clean(self):
        self.assertTrue('__class__' not in self.test_ns.nonprivate)
        self.assertTrue('_private_method' not in self.test_ns.nonprivate)

    def test_methods(self):
        self.assertTrue([self.test_obj._private_method],
                        self.test_ns.methods.values())

    def test_startswith(self):
        self.assertEqual(self.test_ns.startswith('pub').keys(),
                         ['public_static_method'])

    def test_private(self):
        diff = set(self.test_ns.keys()) - set(self.test_ns.private.keys())
        self.assertEqual(diff, set(['class_variable','public_static_method']))

    def test_functions(self):
        self.assertTrue([self.test_obj.public_static_method],
                        self.test_ns.functions.values())

    def test_class_variables(self):
        self.assertTrue(['class_variable'],
                        self.test_ns.class_variables.keys())

if __name__=='__main__':
    main()



""" test_classproperty:
      tests for goulash.classproperty
"""
import unittest

from goulash.classproperty import classproperty

class testing(object):
    @classproperty
    def test(kls): return 3

class TestClassProperty(unittest.TestCase):
    def test_classproperty(self):
        self.assertEqual(testing.test, 3)



""" goulash
"""
from types import FunctionType

is_property           = lambda obj: type(obj).__name__=='property'
is_function           = lambda obj: type(obj)==FunctionType
is_nonprivatefunction = lambda name, obj: (not name.startswith('_')) and is_function(obj)
is_staticmethod       = lambda obj: type(obj).__name__=='staticmethod'


class AllStaticMethods(type):
    """ AllStaticMethods:
         set this class as your metaclass in order to build a
         module-like class.. all methods inside the class will
         be turned into static methods.
    """
    def __init__(*args, **kargs):
        return type.__init__(*args, **kargs)

    def __new__(mcs, name, bases, dct, finished=True):
        """
            NOTE: the 'finished' flag is used for chaining..
                  make sure you know what you're doing if you use it.
        """
        from goulash import Namespace
        for x, func in Namespace(dct).functions.items():
                dct[x] = staticmethod(func)
        if finished:
            return type.__new__(mcs, name, bases, dct)
        else:
            return (mcs, name, bases, dct)



""" goulash.heuristics

    Heuristics typically return answers of some kind, but if the heuristics
    themselves are complex or cascading, simply returning booleans or integers
    or whatever can be confusing.  The objects in this file provide tools
    for wrapping results in such a way that they function as expected but
    can optionally include explanations.  More detailed descriptions of available
    wrappers follow.

    Answer(obj):
      has a truth value depending on the object itself.

    ExplainedAnswer(obj, optional_explanation):
      Couples an answer with an explanation.
      The explanation may be any object, but
      whatever it is will be converted to a string.

    NotApplicable(optional_explanation):
      Heuristic was not applicable.
      This always tests as False.

    Affirmative(explanation):
      An ExplainedAnswer that always tests as true.
      Again any object may be used as the explanation,
      but it will always be converted to a string.

    Negative(explanation):
      An ExplainedAnswer that always tests as false.
      Again any object may be used as the explanation,
      but it will always be converted to a string.

"""

from goulash.wrappers import DumbWrapper

class Answer(DumbWrapper):
    def __str__(self):
        return "({0}: {1})".format(
            self.__class__.__name__,
            str(self.obj))

    def __nonzero__(self):
        return bool(self.obj)

    __repr__ = __str__

class ExplainedAnswer(Answer):
    def __init__(self, obj, explanation="No explanation given."):
        super(ExplainedAnswer, self).__init__(obj)
        self.explanation = str(explanation)

    def __str__(self):
        return "({0}: {1})".format(
            self.__class__.__name__,
            str(self.explanation))

class NotApplicable(DumbWrapper):
    def __init__(self, obj=None):
        super(NotApplicable, self).__init__(obj)

    def __nonzero__(self):
        return False

    def __str__(self):
        return "(NotApplicable: {0})".format(str(self.obj))
    __repr__=__str__

class Affirmative(ExplainedAnswer):
    def __init__(self, explanation="no reason given"):
        self.obj = True
        self.explanation = str(explanation)

    def __nonzero__(self):
        return True

class NegativeAnswer(ExplainedAnswer):
    def __init__(self, explanation="no reason given"):
        self.obj = False
        self.explanation = str(explanation)
Negative = NegativeAnswer



""" goulash.tests.test_decorators
"""

from unittest import TestCase
from goulash.decorators import arg_types, memoized_property

class TestArgTypesDecorator(TestCase):
    def setUp(self):
        self.fxn = lambda x: x

    def test_ignores_kargs(self):
        @arg_types(int)
        def fxn(*args, **kargs):
            return kargs
        fxn(1, 2, 3, foo='bar')

    def test_one_type_bad(self):
        tmp = arg_types(int)(self.fxn)
        self.assertRaises(
            arg_types.ArgTypeError,
            lambda: tmp('3'))

    def test_two_types_bad(self):
        tmp = arg_types(int,list)(self.fxn)
        self.assertRaises(
            arg_types.ArgTypeError,
            lambda: tmp('3'))

    def test_good_two_types(self):
        tmp = arg_types(int, list)(self.fxn)
        tmp(42)

    def test_good_one_type(self):
        tmp = arg_types(list)(self.fxn)
        tmp([])

class TestMemoizedProperty(TestCase):
    def setUp(self):
        self.call_count = 0
        class ExampleClass(object):
            @memoized_property
            def example(himself):
                self.call_count += 1
                return 3
        self.kls = ExampleClass

    def test_memoized_property(self):
        tmp1 = self.kls()
        tmp2 = self.kls()
        self.assertEqual(tmp1.example, 3)
        self.assertEqual(self.call_count, 1)
        self.assertEqual(tmp1.example, 3)
        self.assertEqual(self.call_count, 1)
        #note: different instances *should* get fresh calls
        self.assertEqual(tmp2.example, 3)
        self.assertEqual(self.call_count, 2)
        self.assertEqual(tmp1.example, 3)
        self.assertEqual(tmp2.example, 3)
        self.assertEqual(self.call_count, 2)



""" goulash.tests.data
"""

class TestObject(object):
    """ plain object with some misc stuff. """
    class_variable = 'foo'

    def _private_method(self): pass

    _private_method.test_attr='bar'

    @staticmethod
    def public_static_method(foo, bar): pass

    @property
    def _some_private_property(self):
        return 3

class ComplexTestObject(TestObject):
    """ this object has some inheritance!

        (that's useful for testing 'local' names.)
    """
    my_class_variable = 'foo'



""" goulash._os
"""
def get_mounts_by_type(mtype):
    import os
    tmp = os.popen('mount -l -t {0}'.format(mtype))
    tmp = tmp.readlines()
    tmp = [x.strip() for x in tmp if x.strip()]
    tmp2 = []
    for line in tmp:
        mdata = dict(line=line)
        line = line.split(' on ')
        name = line.pop(0)
        line = ''.join(line)
        line = line.split(' type ')
        mount_point = line.pop(0)
        mdata.update(name=name, mount_point=mount_point)
        tmp2.append(mdata)
    return tmp2



""" goulash.tests.test_metaclasses
"""
import unittest
from inspect import isclass
from goulash.metaclasses import ClassAlgebra


class TestClass(object):
    __metaclass__ = ClassAlgebra
    def test_method(self):
        return 'base-method'

class Mixin(object):
    def mixin_method(self):
        return 'unique-mixin-method'
    def test_method(self):
        return 'mixin-override'

class TestClassAlgebra(unittest.TestCase):
    def setUp(self):
        self.kls = TestClass
        self.mixin = Mixin

    def assertIsClass(self, other, err=None):
        err = err or 'expected {0} would be a class object'.format(other)
        return self.assertTrue(isclass(other), err)
    assertClass = assertIsClass

    def test_subclass(self):
        new_method = lambda himself: 42
        new_kls = self.kls.subclass('MySubclass',new_method=new_method)
        err = 'expected .subclass() would result in a new class'
        self.assertClass(new_kls, err)
        err = 'expected result of .subclass() would support class-algebra'
        self.assertEqual(type(new_kls), ClassAlgebra, err)
        err = 'expected new_method() would work as passed!'
        self.assertEqual(new_kls().new_method(), 42)

    def test_right_mix(self):
        tmp = TestClass >> Mixin
        self.assertClass(tmp)
        expected = (TestClass, Mixin)
        actual = tmp.__bases__
        err = 'expected right-mix would result in bases: ' + str(expected)
        self.assertEqual(expected, actual, err)
        err = 'MRO is broken with right-mix operation'
        self.assertEqual(tmp().test_method(), 'base-method', err)
        self.assertEqual(tmp().mixin_method(), 'unique-mixin-method')

    def test_left_mix(self):
        tmp = TestClass << Mixin
        expected = (Mixin, TestClass)
        actual = tmp.__bases__
        err = 'expected left-mix would result in bases: ' + str(expected)
        self.assertEqual(expected, actual, err)
        err = 'MRO is broken with lefte-mix operation'
        self.assertEqual(tmp().test_method(), 'mixin-override', err)
        self.assertEqual(tmp().mixin_method(), 'unique-mixin-method')


if __name__=='__main__':
    unittest.main()



""" test_ansi.py """
import unittest

from goulash.ansi import Ansi2HtmlConverter


class TestAnsi(unittest.TestCase):
    def setUp(self):
        self.ansi = Ansi2HtmlConverter()

    def test_basic(self):
        self.assertTrue(self.ansi.get_style())
        self.assertEqual('foo', self.ansi.convert('foo'))



""" goulash.tests.test_net

    WARNING: mocking sockets is sketchy as far portability and producing
             useful tests, so lots of these are integration tests, not
             unittests. (i.e. they might fail without a working network)
"""

import mock
import unittest
from goulash import net

class TestNet(unittest.TestCase):

    @mock.patch('socket.gethostbyname')
    def test_ipaddr_basic(self, get_host_by_name):
        get_host_by_name.return_value = 'fake'
        self.assertEqual(set(['fake']), net.ipaddr_basic())

    def test_is_port_open(self):
        self.assertFalse(net.is_port_open('fake-host', 22))

    def test_ipaddr_with_LAN(self):
        # super primitive test:
        # uses the network AND assumes ipv4
        ip = net.ipaddr_with_LAN()
        dots = [x for x in ip if x=='.']
        self.assertEqual(len(dots), 3)

    def test_ipaddr_hosts(self):
        hostname,aliases,ips = net.ipaddr_hosts()
        self.assertTrue(isinstance(hostname, basestring))
        self.assertTrue(isinstance(aliases, list))
        self.assertTrue(isinstance(ips, list))



#!/usr/bin/env python
""" setup.py for goulash
"""
from setuptools import setup, find_packages
setup(
    name         = 'goulash',
    version      = '0.1',
    description  = 'toolbox, random shared stuff from my other projects',
    author       = 'mattvonrocketstein',
    author_email = '$author@gmail',
    url          = 'one of these days',
    download_url = 'https://github.com/mattvonrocketstein/goulash/tarball/0.1',
    package_dir  = {'': 'lib'},
    packages     = find_packages('lib'),
    keywords     = ['goulash'],
    )



""" goulash.wrappers
"""

class JSONWrapper(object):
    # convenience wrapper that makes __getattr__
    # work via __getitem__
    def __init__(self, data):
        self._data = data

    def __getattr__(self, name):
        try:
            return self._data[name]
        except KeyError:
            return getattr(self._data, name)
    __getitem__ = __getattr__

class DumbWrapper(object):
    # Simplest wrapper pattern
    def __init__(self, obj):
        self.obj = obj

    def __getattr__(self, x):
        return getattr(self.obj, x)

    def __iter__(self):
        return iter(self.obj)



""" test_parsing """

import unittest

from goulash import parsing

class TestAnsi(unittest.TestCase):

    def setUp(self):
        pass

    def test_strip_tags(self):
        self.assertEqual(parsing.strip_tags('<div>foo</div>'),'foo')

    def test_sanitize(self):
        self.assertEqual(parsing.sanitize_txt('A B C'), 'a_b_c')

    def test_smart_split(self):
        tmp = parsing.smart_split('a_b-c.d')
        self.assertEqual(tmp,'a b c d'.split())
